@@ -12,6 +12,7 @@
 
 #ifdef HAVE_PCAP_H
 #ifdef HAVE_LIBPCAP
+#include "ldns-dpa.h"
 
 #ifdef HAVE_NETINET_IP6_H
 #include <netinet/ip6.h>
@@ -2764,7 +2765,7 @@ int main(int argc, char *argv[]) {
 	
 	while ((cur = pcap_next(pc, &cur_hdr))) {
 		if (verbosity >= 5) {
-			printf("\n\n\n[PKT_HDR] caplen: %u \tlen: %u\n", cur_hdr.caplen, cur_hdr.len);
+			printf("\n\n\n[PKT_HDR] caplen: %u \tlen: %u\n", (unsigned int)cur_hdr.caplen, (unsigned int)cur_hdr.len);
 		}
 		handle_ether_packet(cur, cur_hdr, count, expr, uniques, unique_ids, unique_id_count);
 	}
@@ -2822,13 +2823,13 @@ int main(int argc, char *argv[]) {
 }
 
 #else
-int main() {
+int main(void) {
 	fprintf(stderr, "ldns-dpa was not built because there is no pcap library on this system, or there was no pcap header file at compilation time. Please install pcap and rebuild.\n");
 	return 1;
 }
 #endif
 #else
-int main() {
+int main(void) {
 	fprintf(stderr, "ldns-dpa was not built because there is no pcap library on this system, or there was no pcap header file at compilation time. Please install pcap and rebuild.\n");
 	return 1;
 }
