@@ -66,14 +66,12 @@ main(int argc, char *argv[])
 	ldns_key *key;
 	ldns_rr *ds;
 
-	ldns_lookup_table *lt;
-
 	prog = strdup(argv[0]);
 	algorithm = 0;
 	random = NULL;
 	ksk = false; /* don't create a ksk per default */
 
-	while ((c = getopt(argc, argv, "a:kb:r:v25")) != -1) {
+	while ((c = getopt(argc, argv, "a:kb:r:v")) != -1) {
 		switch (c) {
 		case 'a':
 			if (algorithm != 0) {
@@ -84,13 +82,7 @@ main(int argc, char *argv[])
 				show_algorithms(stdout);
 				exit(EXIT_SUCCESS);
 			}
-			lt = ldns_signing_algorithms;
-			while (lt->name) {
-				if (strncasecmp(lt->name, optarg, strlen(lt->name)) == 0) {
-					algorithm = lt->id;
-				}
-				lt++;
-			}
+			algorithm = ldns_get_signing_algorithm_by_name(optarg);
 			if (algorithm == 0) {
 				fprintf(stderr, "Algorithm %s not found\n", optarg);
 				show_algorithms(stderr);
@@ -154,6 +146,18 @@ main(int argc, char *argv[])
 			exit(1);
 		}
 		break;
+#ifdef USE_GOST
+	case LDNS_SIGN_ECC_GOST:
+		if(!ldns_key_EVP_load_gost_id()) {
+			fprintf(stderr, "error: libcrypto does not provide GOST\n");
+			exit(EXIT_FAILURE);
+		}
+		break;
+#endif
+#ifdef USE_ECDSA
+	case LDNS_SIGN_ECDSAP256SHA256:
+	case LDNS_SIGN_ECDSAP384SHA384:
+#endif
 	case LDNS_SIGN_HMACMD5:
 	case LDNS_SIGN_HMACSHA1:
 	case LDNS_SIGN_HMACSHA256:
@@ -178,7 +182,7 @@ main(int argc, char *argv[])
 	/* generate a new key */
 	key = ldns_key_new_frm_algorithm(algorithm, bits);
 
-	/* set the owner name in the key - this is a /seperate/ step */
+	/* set the owner name in the key - this is a /separate/ step */
 	ldns_key_set_pubkey_owner(key, domain);
 
 	/* ksk flag */
@@ -194,24 +198,32 @@ main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 	owner = ldns_rdf2str(ldns_rr_owner(pubkey));
-	
+
 	/* calculate and set the keytag */
 	ldns_key_set_keytag(key, ldns_calc_keytag(pubkey));
 
 	/* build the DS record */
 	switch (algorithm) {
-	default:
-		ds = ldns_key_rr2ds(pubkey, LDNS_SHA1);
+#ifdef USE_ECDSA
+	case LDNS_SIGN_ECDSAP384SHA384:
+		ds = ldns_key_rr2ds(pubkey, LDNS_SHA384);
 		break;
+	case LDNS_SIGN_ECDSAP256SHA256:
+#endif
 	case LDNS_SIGN_RSASHA256:
 	case LDNS_SIGN_RSASHA512:
 		ds = ldns_key_rr2ds(pubkey, LDNS_SHA256);
 		break;
+	case LDNS_SIGN_ECC_GOST:
 #ifdef USE_GOST
-	case LDNS_SIGN_GOST:
-		ds = ldns_key_rr2ds(pubkey, LDNS_HASH_GOST94);
-		break;
+		ds = ldns_key_rr2ds(pubkey, LDNS_HASH_GOST);
+#else
+		ds = ldns_key_rr2ds(pubkey, LDNS_SHA256);
 #endif
+		break;
+	default:
+		ds = ldns_key_rr2ds(pubkey, LDNS_SHA1);
+		break;
 	}
 
 	/* print the public key RR to .key */
@@ -227,11 +239,14 @@ main(int argc, char *argv[])
 		LDNS_FREE(filename);
 		exit(EXIT_FAILURE);
 	} else {
+		/* temporarily set question so that TTL is not printed */
+		ldns_rr_set_question(pubkey, true);
 		ldns_rr_print(file, pubkey);
+		ldns_rr_set_question(pubkey, false);
 		fclose(file);
 		LDNS_FREE(filename);
 	}
-	
+
 	/* print the priv key to stderr */
 	filename = LDNS_XMALLOC(char, strlen(owner) + 21);
 	snprintf(filename, strlen(owner) + 20, "K%s+%03u+%05u.private", owner, algorithm, (unsigned int) ldns_key_keytag(key));
@@ -249,7 +264,7 @@ main(int argc, char *argv[])
 		fclose(file);
 		LDNS_FREE(filename);
 	}
-	
+
 	/* print the DS to .ds */
 	if (algorithm != LDNS_SIGN_HMACMD5 &&
 		algorithm != LDNS_SIGN_HMACSHA1 &&
@@ -266,11 +281,14 @@ main(int argc, char *argv[])
 			LDNS_FREE(filename);
 			exit(EXIT_FAILURE);
 		} else {
+			/* temporarily set question so that TTL is not printed */
+			ldns_rr_set_question(ds, true);
 			ldns_rr_print(file, ds);
+			ldns_rr_set_question(ds, false);
 			fclose(file);
 			LDNS_FREE(filename);
 		}
-	}	
+	}
 
 	fprintf(stdout, "K%s+%03u+%05u\n", owner, algorithm, (unsigned int) ldns_key_keytag(key));
 	ldns_key_deep_free(key);
