@@ -236,8 +236,10 @@ main(int argc, char **argv)
 		ldns_rr_list_free(answer_ad);
 	}
 	
-	ldns_rdf_deep_free(origin);
-	ldns_zone_deep_free(zone);
-	
-	return 0;
+	/* No cleanup because of the infinite loop
+	 *
+	 * ldns_rdf_deep_free(origin);
+	 * ldns_zone_deep_free(zone);
+	 * return 0;
+	 */
 }
